// Persistence of Vision Ray Tracer Include File
// Part of the IRTC GIP object archives
// Version: 1.0

// Macro: IR_Canal(CanalWidth, CanalLength, StreetLevel, WaterLevel)

// Object Description: Two walls for lining a canal
// Email: ir@ibras.dk
// Web page: http://www.ibras.dk/
// Date (yyyy-mm-dd): 2000.01.25

// Origin: Left end of canal, at mid-canal.
// Orientation: Y = up
// Dimensions: 1 unit = 1 cm
// Size: x-axis: -15 to CanalLength+15
//       y-axis: StreetLevel - 300 to StreetLevel + 112
//       z-axis: -42 - CanalWidth/2 to 42 + CanalWidth/2
                      
// Includes required: colors.inc, stones1.inc
// Example: object { IR_Canal(800, 10000, 175, -50) }
// Variables: 

// Comments: Instant canal - just add water.
//           The canal railing runs from runs along the x-axis from
//           0 to CanalLength. The wall extends 15 cm beyond that at
//           both ends. The CanalLength should be a multiple of 100.
//           CanalWidth gives the distance between the two walls in
//           the z-axis.
//           The top of the walls are translated to StreetLevel.
//           WaterLevel is used to place a band of algae just above
//           the water line.
//           A water surface has to be added by the calling scene file
//           at WaterLevel.

// Scaling ///////////////////////////////////////////////////// 
// 1 cm = 1 baseUnit 
#ifndef(baseUnit) #declare baseUnit = 1; #end 
/////// DO NOT CHANGE ANYTHING BELOW ///////////////////////////// 
// Metric 
#ifndef (m) /***meter****/ #local m  = 100 *baseUnit; #end 
#ifndef(cm) /*centimeter*/ #local cm = 1.0 *baseUnit; #end 
#ifndef(mm) /*millimeter*/ #local mm = 0.1 *baseUnit; #end 
// English 
#ifndef(in) /*inch*/ #local in = 2.54 *baseUnit; #end 
#ifndef(ft) /*foot*/ #local ft = 12*in*baseUnit; #end 
#ifndef(yd) /*yard*/ #local yd = 3*ft *baseUnit; #end 
////////////////////////////////////////////////////////////////// 

global_settings { assumed_gamma 1.0 } 

/*********************** Control Variables ***************************/

// Global

#ifndef (ShowDetail)
   #local ShowDetail=yes;
#end   

// Local     
   
/*-------------------------- Textures -------------------------------*/

#local VeryVeryDarkBrown=color red 0.03 green 0.01 blue 0.00;

#declare T_CSJern=texture{
   pigment { VeryVeryDarkBrown }
#if(ShowDetail=1)
   finish {phong 0.05  phong_size 10}
   normal { bumps 0.1 scale <0.5, 0.5, 1> }
#end
}

#declare T_CSStone=texture {
#if (ShowDetail = 1)
   brick texture { pigment { Gray10 } }, texture {T_Stone6 scale 10} 
   brick_size <100, 50, 100>
   mortar 1.8
#else
   pigment { Gray20 }
#end   
}                

#declare T_CSTopStone=texture {
#if (ShowDetail = 1)
   brick texture { pigment { Gray10 } }, texture {T_Stone10 } 
   brick_size <150, 5000, 100>
   mortar 2
#else
   pigment { Gray20 }
#end   
}                

#declare T_CSAlger=texture {
   pigment { gradient y
      color_map{
         [0.00, 0.09  color rgbf <0.5, 0.7, 0.5, 0.8>
	                   color rgbf <1, 1,   1, 1> ]
         [0.08, 1.001 color rgbf <1, 1,   1, 1>
	                   color rgbf <1, 1,   1, 1> ]
	   }               
   } 
   scale <1, 300, 1> 
   translate <0, -600, 0> 
}   

/*------------------------- Objects ----------------------------------*/


#declare CanalRailingPost=union{
   box  { <-0.3, 0, -2>, <0.3, 60, 2> }
   difference {
      cylinder { <-0.3, 60, -40>, <0.3, 60, -40>, 42 }
      cylinder { <-1, 60, -40>, <1, 60, -40>, 38 }
      plane { y, 0 translate <0, 60, 0> }
      plane { z, 0 translate <0, 0, -40> }
   }  
   scale <1, 1, -1> 
   texture { T_CSJern }
}

#macro CanalRailing(CWLength)
union {
   #local pos = 0;
   #while (pos <= CWLength)
      object { CanalRailingPost translate <pos, 0, 0> }
      #local pos = pos + 100;
   #end
   cylinder { <0, 55, 0>, <CWLength, 55, 0>, 1.5 }
   cylinder { <0, 100, 38>, <CWLength, 100, 38>, 1.5 }
   texture { T_CSJern }
}
#end
                         
#declare CanalTopStone=union {
   box { <0, 0, 0>, <1, 10, 45> }
   cylinder { <0, 5, 0>, <1, 5, 0>, 5 }
   cylinder { <0, 5, 45>, <1, 5, 45>, 5 }
}
   
#macro IR_CanalWall(CWLength, StreetLevel, WaterLevel)
union {
   box { <-15, 0, 0>, <CWLength+15, -300, 50> texture { T_CSStone } }     
   box { <-15, 0, -0.02>, <CWLength+15, -300, -0.01> texture { T_CSAlger translate <0, WaterLevel-StreetLevel, 0>} }
   object { CanalTopStone scale <CWLength+30, 1, 1> texture { T_CSTopStone } }
   object { CanalRailing(CWLength) translate <0, 10, 25> }
   translate <0, StreetLevel, 0>
}    
#end

/*---------------- Assembled Object -------------------*/

#macro IR_Canal(CanalWidth, CanalLength, StreetLevel, WaterLevel)
union {
   object { IR_CanalWall(CanalLength, StreetLevel, WaterLevel) translate <0, 0, CanalWidth*0.5> }
   object { IR_CanalWall(CanalLength, StreetLevel, WaterLevel) rotate <0, -180, 0> translate <CanalLength, 0, -CanalWidth*0.5> }
   #declare IR_Canal_x = CanalLength + 30;
   #declare IR_Canal_y = 422;
   #declare IR_Canal_z = CanalWidth + 84;
}
#end         
